"""
Module containing the spacing functions for the binarycpython package. Very under-populated at the moment, but more are likely to come soon

Tasks:
    TODO: add more spacing functions to this module.
"""

from typing import Union
import numpy as np
import math

def const(
    min_bound: Union[int, float], max_bound: Union[int, float], steps: int
) -> list:
    """
    Samples a range linearly. Uses numpy linspace.

    Args:
        min_bound: lower bound of range
        max_bound: upper bound of range
        steps: number of segments between min_bound and max_bound

    Returns:
        np.linspace(min_bound, max_bound, steps+1)
    """

    return np.linspace(min_bound, max_bound, steps+1)

############################################################
def peak_normalized_gaussian_func(
    x: Union[int, float], mean: Union[int, float], sigma: Union[int, float]
) -> Union[int, float]:
    """
    Function to evaluate a Gaussian at a given point, note
    that the normalization is such that the peak is always 1.0,
    not that the integral is 1.0

    Args:
        x: location at which to evaluate the distribution
        mean: mean of the Gaussian
        sigma: standard deviation of the Gaussian

    Returns:
        value of the Gaussian at x
    """
    gaussian_prefactor = 1.0 #/ math.sqrt(2.0 * math.pi)

    r = 1.0 / sigma
    y = (x - mean) * r
    return math.exp(-0.5 * y ** 2)

############################################################
def gaussian_zoom(
        min_bound: Union[int, float],
        max_bound: Union[int, float],
        zoom_mean: Union[int, float],
        zoom_dispersion: Union[int,float],
        zoom_magnitude: Union[int,float],
        steps: int
) -> list:
    """
    Samples such that a region is zoomed in according to a 1-Gaussian function

    Args:
        min_bound: lower bound of range
        max_bound: upper bound of range
        zoom_mean: mean of the Gaussian zoom location
        zoom_dispersion: dispersion of the Gaussian
        zoom_magnitude: depth of the Gaussian (should be 0<= zoom_magntiude <1)
        steps: number of segments between min_bound and max_bound assuming a linear step
               this is what you'd normally call "resolution"

    Returns:
        np.linspace(min_bound, max_bound, steps+1)
    """

    # linear spacing: this is what we'd have
    # in the absence of a Gaussian zoom
    linear_spacing = (max_bound - min_bound)/(steps-1)

    # make the list of values
    x = min_bound
    array = np.array([])
    while x <= max_bound:
        array = np.append(array,x)
        g = peak_normalized_gaussian_func(x,zoom_mean,zoom_dispersion)
        f = 1.0 - zoom_magnitude * g
        dx = linear_spacing * f
        x = x + dx

    # force the last array member to be max_bound if it's not
    if array[-1] != max_bound:
        array[-1] = max_bound

    return array
